/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

/**
 *
 */
package org.netbeans.modules.bpel.model.impl;

import java.util.List;

import org.netbeans.modules.bpel.model.api.BpelEntity;
import org.netbeans.modules.bpel.model.api.Source;
import org.netbeans.modules.bpel.model.api.SourceContainer;
import org.netbeans.modules.bpel.model.api.support.BpelModelVisitor;
import org.netbeans.modules.bpel.model.xam.AfterTargets;
import org.netbeans.modules.bpel.model.xam.BpelElements;
import org.w3c.dom.Element;


/**
 * @author ads
 *
 */
public class SourceContainerImpl extends ExtensibleElementsImpl implements
        BpelEntity, SourceContainer, AfterTargets
{

    SourceContainerImpl( BpelModelImpl model, Element e ) {
        super(model, e);
    }

    SourceContainerImpl( BpelBuilderImpl builder ) {
        super(builder, BpelElements.SOURCES.getName() );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.BpelEntity#getElementType()
     */
    public Class<? extends BpelEntity> getElementType() {
        return SourceContainer.class;
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.SourceContainer#getSources()
     */
    public Source[] getSources() {
        readLock();
        try {
            List<Source> list = getChildren( Source.class );
            return list.toArray( new Source[ list.size() ]);
        }
        finally {
            readUnlock();
        }       
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.SourceContainer#getSource(int)
     */
    public Source getSource( int i ) {
        return getChild( Source.class , i );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.SourceContainer#removeSource(int)
     */
    public void removeSource( int i ) {
        removeChild( Source.class , i );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.SourceContainer#setSource(org.netbeans.modules.soa.model.bpel20.api.Source, int)
     */
    public void setSource( Source source, int i ) {
        setChildAtIndex( source , Source.class ,  i );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.SourceContainer#addSource(org.netbeans.modules.soa.model.bpel20.api.Source)
     */
    public void addSource( Source source ) {
        addChild( source , Source.class );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.SourceContainer#setSources(org.netbeans.modules.soa.model.bpel20.api.Source[])
     */
    public void setSources( Source[] source ) {
        setArrayBefore( source , Source.class );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.SourceContainer#insertSource(org.netbeans.modules.soa.model.bpel20.api.Source, int)
     */
    public void insertSource( Source source, int i ) {
        setChildAtIndex( source , Source.class , i );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.SourceContainer#sizeOfSource()
     */
    public int sizeOfSource() {
        readLock();
        try {
            return getChildren( Source.class ).size();
        }
        finally {
            readUnlock();
        }
    }
    

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.impl.BpelEntityImpl#acceptThis(org.netbeans.modules.soa.model.bpel20.api.support.BpelModelVisitor)
     */
    public void accept( BpelModelVisitor visitor ) {
        visitor.visit( this );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.impl.ExtensibleElementsImpl#create(org.w3c.dom.Element)
     */
    @Override
    protected BpelEntity create( Element element ) {
        if ( BpelElements.SOURCE.getName().equals( element.getLocalName() )){
            return new SourceImpl( getModel() , element );
        }
        return super.create(element);
    }

}
