/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.bpel.model.impl;

import java.util.List;

import org.netbeans.modules.bpel.model.api.BpelEntity;
import org.netbeans.modules.bpel.model.api.ToPart;
import org.netbeans.modules.bpel.model.api.ToPartContainer;
import org.netbeans.modules.bpel.model.api.support.BpelModelVisitor;
import org.netbeans.modules.bpel.model.xam.BpelElements;
import org.w3c.dom.Element;


/**
 * @author ads
 *
 */
public class ToPartContainerImpl extends ExtensibleElementsImpl implements
        ToPartContainer
{

    ToPartContainerImpl( BpelModelImpl model, Element e ) {
        super(model, e);
    }

    ToPartContainerImpl( BpelBuilderImpl builder) {
        super(builder, BpelElements.TO_PARTS.getName() );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.ToPartContainer#getToParts()
     */
    public ToPart[] getToParts() {
        readLock();
        try {
            List<ToPart> list = getChildren( ToPart.class );
            return list.toArray( new ToPart[ list.size() ]);
        }
        finally {
            readUnlock();
        }
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.ToPartContainer#getToPart(int)
     */
    public ToPart getToPart( int i ) {
        return getChild( ToPart.class , i );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.ToPartContainer#setToPart(org.netbeans.modules.soa.model.bpel20.api.ToPart, int)
     */
    public void setToPart( ToPart part, int i ) {
        setChildAtIndex( part, ToPart.class , i );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.ToPartContainer#insertToPart(org.netbeans.modules.soa.model.bpel20.api.ToPart, int)
     */
    public void insertToPart( ToPart part, int i ) {
        insertAtIndex( part , ToPart.class , i  );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.ToPartContainer#addToPart(org.netbeans.modules.soa.model.bpel20.api.ToPart)
     */
    public void addToPart( ToPart part ) {
        addChild( part , ToPart.class );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.ToPartContainer#removeToPart(int)
     */
    public void removeToPart( int i ) {
        removeChild( ToPart.class , i );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.ToPartContainer#setToPart(org.netbeans.modules.soa.model.bpel20.api.ToPart[])
     */
    public void setToPart( ToPart[] parts ) {
        setArrayBefore( parts , ToPart.class );
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.soa.model.bpel20.api.ToPartContainer#sizeOfToParts()
     */
    public int sizeOfToParts() {
        readLock();
        try {
            return getChildren( ToPart.class ).size(); 
        }
        finally {
            readUnlock();
        }
    }

    /* (non-Javadoc)
     * @see org.netbeans.modules.bpel.model.api.BpelEntity#getElementType()
     */
    public Class<? extends BpelEntity> getElementType() {
        return ToPartContainer.class;
    }
    

    /* (non-Javadoc)
     * @see org.netbeans.modules.bpel.model.api.BpelEntity#accept(org.netbeans.modules.bpel.model.api.support.BpelModelVisitor)
     */
    public void accept( BpelModelVisitor visitor ) {
        visitor.visit( this );
    }
    

    /* (non-Javadoc)
     * @see org.netbeans.modules.bpel.model.impl.ExtensibleElementsImpl#create(org.w3c.dom.Element)
     */
    @Override
    protected BpelEntity create( Element element )
    {
        if ( BpelElements.TO_PART.getName().equals( element.getLocalName())) {
            return new ToPartImpl( getModel() , element );
        }
        return super.create(element);
    }

}
