/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.bpel.model.impl;

import java.util.List;

import org.netbeans.modules.bpel.model.api.BpelEntity;
import org.netbeans.modules.bpel.model.api.Variable;
import org.netbeans.modules.bpel.model.api.VariableContainer;
import org.netbeans.modules.bpel.model.api.support.BpelModelVisitor;
import org.netbeans.modules.bpel.model.xam.AfterImport;
import org.netbeans.modules.bpel.model.xam.AfterSources;
import org.netbeans.modules.bpel.model.xam.BpelElements;
import org.netbeans.modules.xml.xam.Component;
import org.w3c.dom.Element;

/**
 * @author ads
 */
public class VariableContainerImpl extends ExtensibleElementsImpl implements
        VariableContainer, AfterImport, AfterSources
{

    VariableContainerImpl( BpelModelImpl model, Element e ) {
        super(model, e);
    }

    VariableContainerImpl( BpelBuilderImpl builder ) {
        super(builder, BpelElements.VARIABLES.getName() );
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel20.api.VariableContainer#getVariables()
     */
    public Variable[] getVariables() {
        readLock();
        try {
            List<Variable> list = getChildren(Variable.class);
            return list.toArray(new Variable[list.size()]);
        }
        finally {
            readUnlock();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel20.api.VariableContainer#getVariable(int)
     */
    public Variable getVariable( int i ) {
        return getChild(Variable.class, i);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel20.api.VariableContainer#setVariables(org.netbeans.modules.soa.model.bpel20.api.Variable[])
     */
    public void setVariables( Variable[] variables ) {
        setArrayBefore(variables, Variable.class);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel20.api.VariableContainer#removeVariable(int)
     */
    public void removeVariable( int i ) {
        removeChild(Variable.class, i);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel20.api.VariableContainer#setVariable(org.netbeans.modules.soa.model.bpel20.api.Variable,
     *      int)
     */
    public void setVariable( Variable variable, int i ) {
        setChildAtIndex(variable, Variable.class, i);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel20.api.VariableContainer#addVariable(org.netbeans.modules.soa.model.bpel20.api.Variable)
     */
    public void addVariable( Variable variable ) {
        addChild(variable, Variable.class);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel20.api.VariableContainer#insertVariable(org.netbeans.modules.soa.model.bpel20.api.Variable,
     *      int)
     */
    public void insertVariable( Variable variable, int i ) {
        insertAtIndex(variable, Variable.class, i);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel20.api.VariableContainer#sizeOfVariable()
     */
    public int sizeOfVariable() {
        readLock();
        try {
            return getChildren(Variable.class).size();
        }
        finally {
            readUnlock();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel20.api.BpelEntity#getElementType()
     */
    public Class<? extends BpelEntity> getElementType() {
        return VariableContainer.class;
    }

    public void accept( BpelModelVisitor visitor ) {
        visitor.visit(this);
    }
    
    /* (non-Javadoc)
     * @see org.netbeans.modules.bpel.model.impl.BpelContainerImpl#canPaste(org.netbeans.modules.xml.xam.Component)
     */
    @Override
    public boolean canPaste( Component child )
    {
        boolean flag = super.canPaste(child);
        if ( flag && child instanceof Variable ){
            String name = ((Variable) child).getName();
            if ( name == null ){
                return flag;
            }
            Variable[] variables = getVariables();
            for (Variable variable : variables) {
                if ( name.equals( variable.getName()) ){
                    return false;
                }
            }
        }
        return flag;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.netbeans.modules.soa.model.bpel.xdm.impl.BpelContainerImpl#create(org.w3c.dom.Element)
     */
    @Override
    protected BpelEntity create( Element element )
    {
        if (BpelElements.VARIABLE.getName().equals(element.getLocalName())) {
            return new VariableImpl(getModel(), element);
        }
        return super.create( element );
    }
    
}
