/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */


package org.netbeans.modules.bpel.debugger.test.stub;

import junit.framework.TestCase;
import org.netbeans.modules.bpel.debugger.BreakPosition;
import org.netbeans.modules.bpel.debugger.bdiclient.IBreakViewController;


public class SimulationPositionThread extends Thread {

    private TestCase mTestCase;
    private IBreakViewController mViewController;


    public SimulationPositionThread(TestCase testCase, IBreakViewController viewController) {
        mTestCase = testCase;
        mViewController = viewController;
    }
    
    
    public static SimulationPositionThread[] createSimulationThreads(TestCase testCase, IBreakViewController viewController) {
        SimulationPositionThread[] threads = new SimulationPositionThread[10];
        for (int i=0; i < threads.length; i++) {
            threads[i] = new SimulationPositionThread(testCase, viewController);
        }
        return threads;
    }
    
    
    public static void startThreads(TestCase testCase, SimulationPositionThread[] threads) {
        // start all threads, wait until all have completed
        for (int i=0; i < threads.length; i++) {
            threads[i].start();
        }
        try {
            Thread.sleep(2000);
        } catch (Exception e) {
            testCase.assertNull(e);
        }
    }
    
    
    public void run() {
        yield();
        delay(500);
        yield();
        mViewController.setActivePosition(generateBreakPosition());
    }
    
    private void delay(int ms) {
        try {
            sleep(ms);
        } catch (Exception e) {
            mTestCase.assertNull(e);
        }
    }
    
    private BreakPosition generateBreakPosition() {
        int lineNumber = (int) (Math.random() * 100);
        // generate a random word (using letters a-z)
        String name = String.copyValueOf(new char[] {
                (char) (Math.random() * 25 + 97),
                (char) (Math.random() * 25 + 97),
                (char) (Math.random() * 25 + 97),
                (char) (Math.random() * 25 + 97) });
        BreakPosition bp = new BreakPosition(
                new StubProcessInstance(name, lineNumber, null), null, lineNumber);
        return bp;
    }
}
