/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.bpel.debugger.ui.variable;

import java.util.HashSet;
import org.netbeans.modules.bpel.debugger.api.variables.NamedValueHost;
import org.netbeans.modules.bpel.debugger.api.variables.Value;
import org.netbeans.modules.bpel.debugger.api.variables.Variable;
import org.netbeans.modules.bpel.debugger.api.variables.WsdlMessageValue;
import org.netbeans.modules.bpel.debugger.api.variables.XmlElementValue;

import org.netbeans.spi.viewmodel.TreeExpansionModel;
import org.netbeans.spi.viewmodel.UnknownTypeException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author Alexander Zgursky
 */
public class BpelVariableTreeExpansionModel implements TreeExpansionModel {
    
    private HashSet myExpandedNodes = new HashSet();
    
    //TODO:need to throw UnknownTypeException if we don't support given object
    
    /**{@inheritDoc}*/
    public synchronized boolean isExpanded(Object object) throws UnknownTypeException {
        String key = getKey(object);
        if (key != null) {
            return myExpandedNodes.contains(key);
        } else {
            return false;
        }
    }
    
    /**{@inheritDoc}*/
    public synchronized void nodeExpanded(Object object) {
        String key = getKey(object);
        if (key != null) {
            myExpandedNodes.add(key);
        }
    }
    
    /**{@inheritDoc}*/
    public synchronized void nodeCollapsed(Object object) {
        String key = getKey(object);
        if (key != null) {
            myExpandedNodes.remove(key);
        }
    }
    
    private String getKey(Object object) {
        if (object instanceof Variable) {
            return ((Variable)object).getName();
        } else if (object instanceof WsdlMessageValue.Part) {
            WsdlMessageValue.Part part = (WsdlMessageValue.Part)object;
            NamedValueHost message = part.getMessage().getValueHost();
            return message.getName() + "." + part.getName();
        } else if (object instanceof Element) {
            Element element = (Element)object;
            XmlElementValue value = XmlElementValue.Helper.find(element);
            String varKey = getKey(value.getValueHost());
            return varKey + "/" + XmlElementValue.Helper.xpath(element);
        } else {
            return null;
        }
    }
}
