/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.bpel.xpath.model.node.visitor;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import org.netbeans.modules.bpel.xpath.model.nodes.BooleanPlaceHolderNode;
import org.netbeans.modules.bpel.xpath.model.nodes.BranchesPlaceHolderNode;
import org.netbeans.modules.bpel.xpath.model.nodes.CatchNode;
import org.netbeans.modules.bpel.xpath.model.nodes.DeadlinePlaceHolderNode;
import org.netbeans.modules.bpel.xpath.model.nodes.DurationPlaceHolderNode;
import org.netbeans.modules.bpel.xpath.model.nodes.FinalCounterValuePlaceHolderNode;
import org.netbeans.modules.bpel.xpath.model.nodes.ForEachNode;
import org.netbeans.modules.bpel.xpath.model.nodes.Node;
import org.netbeans.modules.bpel.xpath.model.nodes.PartNode;
import org.netbeans.modules.bpel.xpath.model.nodes.PredicatePlaceHolderNode;
import org.netbeans.modules.bpel.xpath.model.nodes.RootPlaceHolderNode;
import org.netbeans.modules.bpel.xpath.model.nodes.ScopeNode;
import org.netbeans.modules.bpel.xpath.model.nodes.StartCounterValuePlaceHolderNode;
import org.netbeans.modules.bpel.xpath.model.nodes.TreeNode;
import org.netbeans.modules.bpel.xpath.model.nodes.VariableNode;
import org.netbeans.modules.bpel.xpath.model.nodes.StandardRootNode;
import org.netbeans.modules.bpel.xpath.model.nodes.VariablesNode;
import org.netbeans.modules.bpel.xpath.model.nodes.XPathLiteralNode;
import org.netbeans.modules.bpel.xpath.model.nodes.XPathLocationStepNode;
import org.netbeans.modules.bpel.xpath.model.nodes.XPathOperatorNode;
import org.netbeans.modules.bpel.xpath.model.nodes.XPathPredicateNode;
import org.netbeans.modules.bpel.xpath.model.nodes.XPathPredicatesNode;
import org.netbeans.modules.bpel.xpath.model.nodes.XSDAttributeNode;
import org.netbeans.modules.bpel.xpath.model.nodes.XSDComplexTypeNode;
import org.netbeans.modules.bpel.xpath.model.nodes.XSDElementNode;
import org.netbeans.modules.bpel.xpath.model.nodes.XSDSimpleTypeNode;

/**
 * Creates a list of all local variables of the current mapper tree
 * node hierarchy, whatever it may happen to be, sorted in order of
 * precedence. The contents of the list are variable nodes, and if
 * more than one variable exists with the same name then the variable
 * with higher precedence will have a lower list index (i.e. the more
 * local the variable, the earlier it appears in the list).
 * Global variables do not appear in the generated list, this visitor
 * is only for local variables.
 * 
 * @author Josh Sandusky
 */
public class NodeLocalVariablePrecedenceVisitor extends AbstractNodeVisitor {
    
    private Map mPrecedenceMap;
    
    
    public NodeLocalVariablePrecedenceVisitor() {
        mPrecedenceMap = new TreeMap();
    }
    
    public void visit(ScopeNode node) {
        visitTreeNode(node);
    }
    
    public void visit(ForEachNode node) {
        if (node.getChildren().size() == 1 && node.getChild(0) instanceof ScopeNode) {
            ScopeNode scopeNode = (ScopeNode)node.getChild(0);
            visit(scopeNode);
        }
        visitTreeNode(node);
    }

    public void visit(CatchNode node) {
        if (node.getChildren().size() == 1) {
            addVariableNode((VariableNode) node.getChild(0));
        }
        visitTreeNode(node);
    }
    
    public void visit(StandardRootNode node) {
        visitTreeNode(node);
    }

    public void visit(VariablesNode node) {
        visitTreeNode(node);
    }

    public void visit(VariableNode node) {
        addVariableNode(node);
        visitTreeNode(node);
    }
    
    public void visit(PartNode node) {
        visitTreeNode(node);
    }
    
    public void visit(XPathPredicateNode node) {
        visitTreeNode(node);
    }
    
    public void visit(XPathPredicatesNode node) {
        visitTreeNode(node);
    }
    
    public void visit(XPathLocationStepNode node) {
        visitTreeNode(node);
    }
    
    public void visit(BooleanPlaceHolderNode node) {
        visitTreeNode(node);
    }
    
    public void visit(StartCounterValuePlaceHolderNode node) {
        visitTreeNode(node);
    }

    public void visit(FinalCounterValuePlaceHolderNode node) {
        visitTreeNode(node);
    }
    
    public void visit(BranchesPlaceHolderNode node) {
        visitTreeNode(node);
    }
    
    public void visit(DeadlinePlaceHolderNode node) {
        visitTreeNode(node);
    }
    
    public void visit(DurationPlaceHolderNode node) {
        visitTreeNode(node);
    }
    
    public void visit(PredicatePlaceHolderNode node) {
        visitTreeNode(node);
    }

    public void visit(RootPlaceHolderNode node) {
        visitTreeNode(node);
    }
    
    private void addVariableNode(VariableNode node) {
        int depth = 0;
        TreeNode parent = node.getParent();
        while (parent != null) {
            parent = parent.getParent();
            depth++;
        }
        Integer nesting = new Integer(depth);
        List variablesForNestingLevel = (List) mPrecedenceMap.get(nesting);
        if (variablesForNestingLevel == null) {
            variablesForNestingLevel = new ArrayList();
            mPrecedenceMap.put(nesting, variablesForNestingLevel);
        }
        variablesForNestingLevel.add(node);
    }
    
    private void visitTreeNode(TreeNode node) {
        for (Iterator iter=node.getChildren().iterator(); iter.hasNext();) {
            Node child = (Node) iter.next();
            child.accept(this);
        }
    }
    
    public List getVariablePrecedenceList() {
        List precedenceList = new ArrayList();
        for (Iterator iter=mPrecedenceMap.keySet().iterator(); iter.hasNext();) {
            Integer nesting = (Integer) iter.next();
            List varsAtNesting = (List) mPrecedenceMap.get(nesting);
            precedenceList.addAll(varsAtNesting);
        }
        Collections.reverse(precedenceList);
        return precedenceList;
    }
}

