/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.bpel.xpath.view.expression;

import javax.swing.tree.TreeCellRenderer;
import org.netbeans.modules.bpel.model.api.BpelEntity;
import org.openide.util.NbBundle;


public abstract class TargetTreeCellRendererFactory {
    
    private static final String factoryClassSystemProperty =
        "org.netbeans.modules.bpel.xpath.view.expression.TargetTreeCellRendererFactory";    // NOI18N
    
    private static TargetTreeCellRendererFactory mInstance;
    
    public static TargetTreeCellRendererFactory getInstance() {
        try {
            if (mInstance == null) {
                String factory = System.getProperty(factoryClassSystemProperty,
                    "org.netbeans.modules.bpel.xpath.view.expression.impl.TargetTreeCellRendererFactoryImpl");  // NOI18N
                
                if (factory != null) {
                    Class factoryClass = Class.forName(factory);
                    mInstance = (TargetTreeCellRendererFactory) factoryClass.newInstance();
                } else {
                    throw new RuntimeException(
                            NbBundle.getMessage(TargetTreeCellRendererFactory.class,
                                "STR_NO_CONCRETE_FACTORY_CLASS_SPECIFIED_FOR_X",    // NOI18N
                                factoryClassSystemProperty));
                }
            }
        } catch(Exception ex) {
            throw new RuntimeException(
                    NbBundle.getMessage(TargetTreeCellRendererFactory.class,
                        "STR_FAILED_TO_CREATE_CONCRETE_FACTORY_OF_X",   // NOI18N
                        factoryClassSystemProperty));
        }
        
        return mInstance;
    }
    
    public abstract TreeCellRenderer createAssignUpstreamExpressionTreeCellRenderer(BpelEntity entity);
    
    public abstract TreeCellRenderer createAssignDownstreamExpressionTreeCellRenderer(BpelEntity entity);
    
    public abstract TreeCellRenderer createAssignExpressionTreeCellRenderer();
    
    public abstract TreeCellRenderer createBooleanExpressionTreeCellRenderer();
    
    public abstract TreeCellRenderer createPredicateExpressionTreeCellRenderer();
    
    public abstract TreeCellRenderer createDurationExpressionTreeCellRenderer();
    
    public abstract TreeCellRenderer createDeadlineExpressionTreeCellRenderer();
    
    public abstract TreeCellRenderer createForEachExpressionTreeCellRenderer();
}
