/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.bpel.xpath.view.expression.literal;

import org.netbeans.modules.bpel.xpath.model.nodes.impl.XPathLiteralNodeImpl;
import org.netbeans.modules.bpel.xpath.view.expression.impl.MapperView;
import org.netbeans.modules.soa.mapper.basicmapper.util.MapperUtilities;
import org.netbeans.modules.soa.mapper.common.IMapperEvent;
import org.netbeans.modules.soa.mapper.common.IMapperGroupNode;

import org.netbeans.modules.soa.mapper.common.basicmapper.literal.ILiteralUpdater;
import org.netbeans.modules.soa.mapper.common.basicmapper.literal.ILiteralUpdaterFactory;
import org.netbeans.modules.soa.mapper.common.basicmapper.methoid.IFieldNode;
import org.openide.util.NbBundle;


/**
 * Determines the correct type of xpath model expression updater
 * for each literal type.
 *
 * @author jsandusky
 */
public class BpelLiteralHandler 
implements ILiteralUpdaterFactory {
    
    private BpelLiteralStringUpdater  mStringUpdater;
    private BpelLiteralNumericUpdater mNumericUpdater;
    private MapperView mMapper;
    private BpelLiteralUpdater.XPathNodeExpressionUpdater mXPathNodeExpressionUpdater;
    
    
    public BpelLiteralHandler(MapperView mapper) {
        mMapper = mapper;
    }
    
    
    public ILiteralUpdater getStringUpdater() {
        if (mStringUpdater == null) {
            mStringUpdater = new BpelLiteralStringUpdater();
            mStringUpdater.setXPathProcessor(getExpressionUpdater());
        }
        return mStringUpdater;
    }
    
    public ILiteralUpdater getNumericUpdater() {
        if (mNumericUpdater == null) {
            mNumericUpdater = new BpelLiteralNumericUpdater();
            mNumericUpdater.setXPathProcessor(getExpressionUpdater());
        }
        return mNumericUpdater;
    }
    
    private BpelLiteralUpdater.XPathNodeExpressionUpdater getExpressionUpdater() {
        if (mXPathNodeExpressionUpdater == null) {
            mXPathNodeExpressionUpdater = new BpelLiteralUpdater.XPathNodeExpressionUpdater() {
                public void updateNodeExpression(IFieldNode sourceFieldNode) {
                    IMapperGroupNode groupNode = sourceFieldNode.getGroupNode();
                    mMapper.getBasicMapper().getMapperViewManager().postMapperEvent(
                        MapperUtilities.getMapperEvent(
                            this,
                            groupNode,
                            IMapperEvent.REQ_UPDATE_NODE,
                            NbBundle.getMessage(BpelLiteralHandler.class,
                                "STR_X_REQUESTING_UPDATE_NODE_Y",   // NOI18N
                                getClass().getName(), groupNode)));
                }
            };
        }
        return mXPathNodeExpressionUpdater;
    }
    
    /**
     * Returns the literal updater corresponding to the specified type.
     */
    public ILiteralUpdater createLiteralUpdater(String type) {
        ILiteralUpdater updater = null;
        if (type != null) {
            if (type.equals("number")) {    // NOI18N
                updater = getNumericUpdater();
            } else if (type.equals("string")) { // NOI18N
                updater = getStringUpdater();
            }
        }
        return updater;
    }
}
