/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.bpel.xpath.view.impl;

import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;
import org.netbeans.modules.bpel.model.api.BpelEntity;
import org.netbeans.modules.bpel.model.api.BpelModel;

import org.netbeans.modules.soa.mapper.common.basicmapper.IBasicMapper;
import org.netbeans.modules.soa.mapper.common.IMapperLink;
import org.netbeans.modules.bpel.model.api.ForEach;
import org.netbeans.modules.bpel.xpath.model.node.visitor.XPathGeneratorVisitor;
import org.netbeans.modules.bpel.xpath.model.nodes.Node;
import org.netbeans.modules.bpel.xpath.model.nodes.TreeNode;
import org.netbeans.modules.bpel.xpath.view.AbstractXPathExpressionUpdater;
import org.netbeans.modules.bpel.xpath.view.expression.impl.MapperUtil;
import org.netbeans.modules.soa.mapper.common.basicmapper.tree.IMapperTreeNode;

/**
 *
 * @author jsandusky
 *
 */
public class ForEachXPathExpressionUpdater extends AbstractXPathExpressionUpdater {
    
    private static final Logger LOGGER = Logger.getLogger(ForEachXPathExpressionUpdater.class.getName());
   
    
    ForEachXPathExpressionUpdater(IBasicMapper mapper, BpelEntity holdingEntity) {
        super(mapper, holdingEntity);
    }
    public void handleLinkRemovalFromTargetTreeNode(List links) {
        // no special handling required
    }
    
    public void handleLinkUpdateToNonTargetTreeNode(List links) {
        updateLinkObject(links);
    }
    
    public void handleNodeUpdateNotConnectedToTargetTreeNode(List links) {
        updateLinkObject(links);
    }
    
    private void updateLinkObject(List links) {
        Iterator it = links.iterator();
        while (it.hasNext()) {
            IMapperLink link = (IMapperLink) it.next();
            ForEach forEach = (ForEach) link.getLinkObject();
            
            IMapperTreeNode destTreeNode = (IMapperTreeNode) link.getEndNode();
            if (destTreeNode.isDestTreeNode() && destTreeNode.getPath() != null) {
                Node targetTreeNode = (Node) destTreeNode.getPath().getLastPathComponent();
                if (forEach != null && targetTreeNode instanceof TreeNode) {
                    Node opNode = MapperUtil.getMapperNodeObject(link.getStartNode());
                    if (opNode != null) {
                        XPathGeneratorVisitor xGenerator = new XPathGeneratorVisitor(getBPELElement());
                        opNode.accept(xGenerator);
                        String expression = xGenerator.getXPath();

                        ForEachXPathExpressionBuilder.buildForEachExpression(
                                forEach, 
                                targetTreeNode, 
                                expression, 
                                null);
                        
                        xGenerator.addNewPrefixToProcess();
                    }
                }
            }
        }
    }
}
