/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.sql.framework.evaluators.database.db2v7;

import org.netbeans.modules.sql.framework.evaluators.database.StatementContext;
import org.netbeans.modules.sql.framework.evaluators.database.base.BaseCaseEvaluator;
import org.netbeans.modules.sql.framework.model.SQLCaseOperator;
import org.netbeans.modules.sql.framework.model.SQLObject;

import com.sun.sql.framework.exception.BaseException;

/**
 * Velocity-based evaluator that generates SQL Case expressions with DB2-specific syntax
 * quirks.
 * 
 * @author Jonathan Giron
 * @version $Revision: 1.2 $
 */
public class DB2V7CaseEvaluator extends BaseCaseEvaluator {
    /**
     * Overrides base implementation to properly handle situation where the default action
     * is the null oeprator.
     * 
     * @param operator SQLCaseOperator which will be interrogated to obtain default clause
     * @param context StatementContext to be used in evaluating default action
     * @return DB2-specific default clause
     */
    protected String buildDefaultClause(SQLCaseOperator operator, StatementContext context) throws BaseException {
        String defaultClause = "";

        SQLObject defaultAction = operator.getSQLObject(SQLCaseOperator.DEFAULT);
        if (defaultAction != null) {
            if ("null".equalsIgnoreCase(defaultAction.getDisplayName())) {
                defaultClause = "NULL";
            } else {
                defaultClause = "(" + this.getDB().getEvaluatorFactory().evaluate(defaultAction, context) + ")";
            }
        }
        return defaultClause;
    }
}
